SimpleSelfRebuff:_rev("$Revision: 44957 $")

-------------------------------------------------------------------------------
-- Externals
-------------------------------------------------------------------------------

local SimpleSelfRebuff = SimpleSelfRebuff
local L = AceLibrary("AceLocale-2.2"):new("SimpleSelfRebuff")

local new = SimpleSelfRebuff.new
local del = SimpleSelfRebuff.del
local formatDuration = SimpleSelfRebuff.formatDuration
local colorize = SimpleSelfRebuff.colorize

local STATE_UNSET   = SimpleSelfRebuff.STATE_UNSET
local STATE_BUFFED  = SimpleSelfRebuff.STATE_BUFFED
local STATE_FADING  = SimpleSelfRebuff.STATE_FADING
local STATE_MISSING = SimpleSelfRebuff.STATE_MISSING

-------------------------------------------------------------------------------
-- Locals
-------------------------------------------------------------------------------

local reminderDests = {}
local states
local db

-------------------------------------------------------------------------------
-- Module declaration
-------------------------------------------------------------------------------
	
local Reminder = SimpleSelfRebuff:NewModule('Reminder')

-------------------------------------------------------------------------------
-- Option declaration
-------------------------------------------------------------------------------
	
Reminder.optionName = L["Reminder"]
Reminder.optionDesc = L["Configure the reminder."]

local function IsReminderDisabled()
	return not SimpleSelfRebuff:IsModuleActive(Reminder)
end

local function SCTDisabled()
	return IsReminderDisabled() or not SCT
end

local function MSBTDisabled()
	return IsReminderDisabled() or not MikSBT
end

local function getReminderColor(key)
	local c = db.colors[key]
	return c.r, c.g, c.b
end

local function setReminderColor(key, r, g, b)
	local c = db.colors[key]
	c.r, c.g, c.b = r, g, b
end

local function isReminderDestinationEnabled(dest)
	return db.destinations[dest]
end

local function toggleReminderDestination(dest, value)
	db.destinations[dest] = value and true or false
end

Reminder.options = {
	colorHeader = {
		type = 'header',
		name = L["Colors"],
		order = 59,
	},
	missingColor = {
		type = 'color',
		name = L["Missing message"],
		desc = L["Set the color of the missing message."],
		get = getReminderColor,
		set = setReminderColor,
		passValue = 'missing',
		disabled = IsReminderDisabled,
		order = 60,
	},
	refreshingColor = {
		type = 'color',
		name = L["Refreshing message"],
		desc = L["Set the color of the refreshing message."],
		get = getReminderColor,
		set = setReminderColor,
		passValue = 'refreshing',
		disabled = function() return IsReminderDisabled() or SimpleSelfRebuff.db.profile.rebuffThreshold == 0 end,
		order = 60,
	},
	--
	destHeader = {
		type = 'header',
		name = L["Displays"],
		order = 99,
	},
	chat = {
		type = 'toggle',
		name = L["Chat Frame"],
		desc = L["Print reminder into chat frame."],
		passValue = "chat",
		get = isReminderDestinationEnabled,
		set = toggleReminderDestination, 
		disabled = IsReminderDisabled,
	},
	uierror = {
		type = 'toggle',
		name = L["Blizzard error frame"],
		desc = L["Print reminder into Blizzard error frame."],
		passValue = "uierror",
		get = isReminderDestinationEnabled,
		set = toggleReminderDestination, 
		disabled = IsReminderDisabled,
	},
	sctText1 = {
		type = 'toggle',
		name = L["SCT: Text #1"],
		desc = L["Show reminder with SCT text #1."],
		passValue = "sctText1",
		get = isReminderDestinationEnabled,
		set = toggleReminderDestination, 				
		disabled = SCTDisabled,
	},
	sctText2 = {
		type = 'toggle',
		name = L["SCT: Text #2"],
		desc = L["Show reminder with SCT text #2."],
		passValue = "sctText2",
		get = isReminderDestinationEnabled,
		set = toggleReminderDestination,				
		disabled = SCTDisabled,
	},
	sctMessage = {
		type = 'toggle',
		name = L["SCT: Message"],
		desc = L["Show reminder as SCT message."],
		passValue = "sctMessage",
		get = isReminderDestinationEnabled,
		set = toggleReminderDestination,	
		disabled = SCTDisabled,
	},
}

-------------------------------------------------------------------------------
-- Module core
-------------------------------------------------------------------------------

function Reminder:OnModuleInitialize()
	self:RegisterDefaults('profile', {
		destinations = {
			chat = true,
			sctMessage = true,
		},
		colors = {
			missing    = { r=0.9, g=0.2, b=0.2 },
			refreshing = { r=0.9, g=0.6, b=0.2 },
		},
	})
	db = self.db.profile
end

function Reminder:OnModuleEnable()
	self:RegisterEvent('SimpleSelfRebuff_StateChanged', 'UpdateState')	
	self:UpdateMikSBTAreas()
	
	states = new()
	for category, state, expected, actual, timeLeft in self.core:IterateCategories(STATE_FADING) do	
		self:UpdateState(category, state, expected, actual, timeLeft)
	end	
end

function Reminder:OnModuleDisable()
	states = del(states)
end

function Reminder:UpdateMikSBTAreas()
	if MikSBT then		
		for index,desc in pairs(MikSBT.GetScrollAreaList()) do
			local index,desc = index,desc
			local key = "mikSBT"..index
			if not self.options[key] then
				self.options[key] = {
					type = 'toggle',
					name = L["MikSBT: %s"]:format(desc),
					desc = L["Show reminder in MikSBT %s frame."]:format(desc),
					passValue = key,
					get = isReminderDestinationEnabled,
					set = toggleReminderDestination,			
					disabled = MSBTDisabled,				
				}
			end
			if not reminderDests[key] then
				reminderDests[key] = function(self, text, color)
					MikSBT.DisplayMessage(text, index, false, color.r * 255, color.g * 255, color.b * 255)
				end
			end
		end		
	end
end

function Reminder:UpdateState(category, newState, expected, actual, timeLeft)
	if newState < STATE_FADING then
		newState = nil
	end
	if newState ~= states[category] then
		if newState and newState > (states[category] or 0) then
			self:PrintMessage(expected, timeLeft)
		end
		states[category] = newState
	end
end

function Reminder:RemindAll()
	for category in pairs(states) do
		local state, expected, actual, timeLeft = category:GetState()
		self:PrintMessage(expected, timeLeft)
	end
end

function Reminder:PrintMessage(buff, timeLeft)
	if not buff or not next(db.destinations) then
		return
	end
	local text, color
	if (timeLeft or 0) > 0 then
		text = L["%s expires in %s."]:format(buff.name, formatDuration(timeLeft))
		color = db.colors.refreshing
	else
		text = L["%s is missing."]:format(buff.name)
		color = db.colors.missing
	end 
	for dest,value in pairs(db.destinations) do
		if value and reminderDests[dest] then
			reminderDests[dest](self, text, color)
		end
	end
end

-------------------------------------------------------------------------------
-- Destinations
-------------------------------------------------------------------------------

function reminderDests:chat(text, color)
	self.core:Print(colorize(text, color.r, color.g, color.b))
end

function reminderDests:uierror(text, color)
	UIErrorsFrame:AddMessage(text, color.r, color.g, color.b)
end

function reminderDests:sctText1(text, color)
	if SCT then
		SCT:DisplayText(text, color, false, "event", 1)
	end
end

function reminderDests:sctText2(text, color)
	if SCT then
		SCT:DisplayText(text, color, false, "event", 2)
	end
end

function reminderDests:sctMessage(text, color)
	if SCT then
		SCT:DisplayMessage(text, color)
	end
end